Python 提供了多种方式来读写文件，其核心在于 `open()` 函数和文件对象的各种方法。 选择哪种方法取决于你的需求和文件类型（文本文件或二进制文件）。 以下是对 Python 文件读写操作的详细解释，涵盖了最佳实践和错误处理。

**1. 打开文件 (`open()` 函数)**

`open()` 函数是所有文件操作的起点。它接受以下参数：

- **`file` (必填):** 文件的路径字符串。可以是相对路径（相对于当前脚本的位置）或绝对路径。
    
- **`mode` (可选):** 指定文件的打开模式。常用的模式包括：
    
    - `"r"`: 读取模式 (默认)。 如果文件不存在，则引发 `FileNotFoundError`。
    - `"w"`: 写入模式。 如果文件存在，则覆盖其内容；如果文件不存在，则创建新文件。
    - `"x"`: 排他创建模式。 如果文件不存在，则创建新文件；如果文件存在，则引发 `FileExistsError`。
    - `"a"`: 追加模式。 如果文件存在，则在文件末尾追加内容；如果文件不存在，则创建新文件。
    - `"b"`: 二进制模式。 用于处理二进制文件，例如图像或音频文件。 可以与其他模式组合，例如 `"rb"`、`"wb"` 等。
    - `"t"`: 文本模式 (默认)。 用于处理文本文件。
    - `"+"`: 更新模式。 允许同时读写文件。 例如，`"r+"` 允许读取和写入，`"w+"` 允许写入和读取（写入会覆盖文件内容）。
- **`encoding` (可选):** 指定文件的编码方式。 对于文本文件，这非常重要，例如 `"utf-8"`、`"latin-1"` 等。 如果不指定，则使用系统默认编码，这可能会导致乱码问题。
    
- **`errors` (可选):** 指定如何处理编码错误。例如，`"ignore"` 忽略错误，`"replace"` 用替代字符替换错误字符。
    

**最佳实践：使用 `with` 语句**

强烈建议使用 `with` 语句来打开文件。 `with` 语句确保文件在代码块执行完毕后自动关闭，即使发生异常。 这避免了资源泄漏，并简化了错误处理。

```python
try:
    with open("my_file.txt", "r", encoding="utf-8") as file:
        # 在这里进行文件操作
        content = file.read()
        print(content)
except FileNotFoundError:
    print("File not found.")
except Exception as e:
    print(f"An error occurred: {e}")
```

**2. 读取文件**

- **`read()`:** 读取整个文件内容到一个字符串。
    
- **`readline()`:** 读取文件的一行内容。 返回一个字符串，包括换行符 `\n`。
    
- **`readlines()`:** 读取文件的所有行到一个列表中，每行是一个列表元素（包括换行符）。
    
- **迭代器:** 文件对象本身就是一个迭代器，可以直接在 `for` 循环中使用，逐行读取文件内容。 这是读取大型文件的最有效方法。
    

```python
with open("my_file.txt", "r", encoding="utf-8") as file:
    # 读取整个文件
    content = file.read()
    print("Read all:", content)

    # 读取一行
    file.seek(0) # 回到文件开头
    line = file.readline()
    print("Readline:", line)

    # 读取所有行
    file.seek(0) # 回到文件开头
    lines = file.readlines()
    print("Readlines:", lines)

    # 迭代读取
    file.seek(0) # 回到文件开头
    for line in file:
        print("Iterate:", line, end="")
```

**3. 写入文件**

- **`write()`:** 将字符串写入文件。
    
- **`writelines()`:** 将字符串列表写入文件。
    

```python
with open("my_file.txt", "w", encoding="utf-8") as file:
    file.write("This is the first line.\n")
    file.write("This is the second line.\n")

with open("my_file.txt", "w", encoding="utf-8") as file:
    lines = ["Line 1\n", "Line 2\n", "Line 3\n"]
    file.writelines(lines)
```

**4. 处理二进制文件**

处理二进制文件时，需要使用 `"b"` 模式。 例如，读取图像文件：

```python
with open("image.jpg", "rb") as file:
    image_data = file.read()
    # 处理 image_data
```

**5. 错误处理**

始终使用 `try...except` 块来处理可能发生的异常，例如 `FileNotFoundError`、`IOError` 等。

### 小项目

要求：
1. 将一个超过200k的文件复制
2. 使用`read()` `open()` `write()` 这几个函数

```python
sf = open("D:\\Desktop\\www_jingdutiao_s_20230803_165510.sql.gz","rb")
df = open("D:\\Desktop\\fuzhis_20230803_165510.sql.gz","wb")
while 1:
   data = sf.read(4096)
   if data == b"":
      break
   df.write(data)
df.close()
sf.close()
```

我们现在来分析这一个代码

将我们的目标压缩包存储在`sf` 这个变量当中，因为变量占据的空间是在内存当中的，所以我们的目标文件太大可能会导致内存占用过高卡死，所以如何解决呢？

我们每次只读4kb，但是4kb肯定包含不了整个文件，所以就用到了我们的 `while`循环

我们首相令他为真（True == 1），到文件写满我们就`break`跳出这个循环

最后关闭这两个文件。